@@ -11,6 +11,7 @@
 
 #include "core/or/circuit_st.h"
 #include "core/or/conflux_st.h"
+#include "core/or/relay_msg_st.h"
 
 typedef struct conflux_t conflux_t;
 typedef struct conflux_leg_t conflux_leg_t;
@@ -24,8 +25,7 @@ typedef struct conflux_leg_t conflux_leg_t;
 /** Helper: Return the number of legs a conflux object has. */
 #define CONFLUX_NUM_LEGS(cfx) (smartlist_len(cfx->legs))
 
-/** A cell for the out-of-order queue.
- * XXX: Consider trying to use packed_cell_t instead here? */
+/** A relay message for the out-of-order queue. */
 typedef struct {
   /**
    * Absolute sequence number of this cell, computed from the
@@ -37,10 +37,10 @@ typedef struct {
    */
   int heap_idx;
 
-  /** The cell here is always guaranteed to have removed its
+  /** The relay message here is always guaranteed to have removed its
    * extra conflux sequence number, for ease of processing */
-  cell_t cell;
-} conflux_cell_t;
+  relay_msg_t *msg;
+} conflux_msg_t;
 
 size_t conflux_handle_oom(size_t bytes_to_remove);
 uint64_t conflux_get_total_bytes_allocation(void);
@@ -54,15 +54,18 @@ circuit_t *conflux_decide_circ_for_send(conflux_t *cfx,
 circuit_t *conflux_decide_next_circ(conflux_t *cfx);
 
 int conflux_process_switch_command(circuit_t *in_circ,
-                               crypt_path_t *layer_hint, cell_t *cell,
-                               relay_header_t *rh);
+                               crypt_path_t *layer_hint,
+                               const relay_msg_t *msg);
 bool conflux_should_multiplex(int relay_command);
-bool conflux_process_cell(conflux_t *cfx, circuit_t *in_circ,
-                          crypt_path_t *layer_hint,
-                          cell_t *cell);
-conflux_cell_t *conflux_dequeue_cell(circuit_t *circ);
+bool conflux_process_relay_msg(conflux_t *cfx, circuit_t *in_circ,
+                               crypt_path_t *layer_hint,
+                               const relay_msg_t *msg);
+conflux_msg_t *conflux_dequeue_relay_msg(circuit_t *circ);
 void conflux_note_cell_sent(conflux_t *cfx, circuit_t *circ,
                             uint8_t relay_command);
+void conflux_relay_msg_free_(conflux_msg_t *msg);
+#define conflux_relay_msg_free(msg) \
+  FREE_AND_NULL(conflux_msg_t, conflux_relay_msg_free_, (msg))
 
 /* Private section starts. */
 #ifdef TOR_CONFLUX_PRIVATE
