@@ -63,7 +63,8 @@ build_link_cell(const conflux_cell_link_t *link, uint8_t *cell_out)
       trn_cell_conflux_link_getlen_payload(cell), payload);
 
   /* Encode cell. */
-  cell_len = trn_cell_conflux_link_encode(cell_out, RELAY_PAYLOAD_SIZE, cell);
+  cell_len = trn_cell_conflux_link_encode(cell_out,
+                                          RELAY_PAYLOAD_SIZE_MAX, cell);
 
   trn_cell_conflux_link_payload_v1_free(payload);
   trn_cell_conflux_link_free(cell);
@@ -87,7 +88,8 @@ build_linked_ack_cell(uint8_t *cell_out)
   tor_assert(cell_out);
 
   cell = trn_cell_conflux_linked_ack_new();
-  cell_len = trn_cell_conflux_linked_ack_encode(cell_out, RELAY_PAYLOAD_SIZE,
+  cell_len = trn_cell_conflux_linked_ack_encode(cell_out,
+                                                RELAY_PAYLOAD_SIZE_MAX,
                                                 cell);
 
   trn_cell_conflux_linked_ack_free(cell);
@@ -97,7 +99,7 @@ build_linked_ack_cell(uint8_t *cell_out)
 bool
 conflux_cell_send_link(const conflux_cell_link_t *link, origin_circuit_t *circ)
 {
-  uint8_t payload[RELAY_PAYLOAD_SIZE] = {0};
+  uint8_t payload[RELAY_PAYLOAD_SIZE_MAX] = {0};
   ssize_t cell_len;
 
   tor_assert(link);
@@ -131,7 +133,7 @@ conflux_cell_send_link(const conflux_cell_link_t *link, origin_circuit_t *circ)
 bool
 conflux_cell_send_linked(const conflux_cell_link_t *link, or_circuit_t *circ)
 {
-  uint8_t payload[RELAY_PAYLOAD_SIZE] = {0};
+  uint8_t payload[RELAY_PAYLOAD_SIZE_MAX] = {0};
   ssize_t cell_len;
 
   tor_assert(link);
@@ -164,7 +166,7 @@ conflux_cell_send_linked(const conflux_cell_link_t *link, or_circuit_t *circ)
 bool
 conflux_cell_send_linked_ack(origin_circuit_t *circ)
 {
-  uint8_t payload[RELAY_PAYLOAD_SIZE] = {0};
+  uint8_t payload[RELAY_PAYLOAD_SIZE_MAX] = {0};
   ssize_t cell_len;
 
   tor_assert(circ);
@@ -226,16 +228,14 @@ conflux_cell_parse_link_v1(const trn_cell_conflux_link_t *trn_link)
 }
 
 conflux_cell_link_t *
-conflux_cell_parse_link(const cell_t *cell, const uint16_t cell_len)
+conflux_cell_parse_link(const relay_msg_t *msg)
 {
   conflux_cell_link_t *link = NULL;
   trn_cell_conflux_link_t *trn_cell = NULL;
 
-  tor_assert(cell);
+  tor_assert(msg);
 
-  if (trn_cell_conflux_link_parse(&trn_cell,
-                                  cell->payload + RELAY_HEADER_SIZE,
-                                  cell_len) < 0) {
+  if (trn_cell_conflux_link_parse(&trn_cell, msg->body, msg->length) < 0) {
     log_fn(LOG_PROTOCOL_WARN, LD_CIRC,
            "Unable to parse CONFLUX_LINK cell.");
     goto end;
@@ -258,10 +258,10 @@ conflux_cell_parse_link(const cell_t *cell, const uint16_t cell_len)
 }
 
 conflux_cell_link_t *
-conflux_cell_parse_linked(const cell_t *cell, const uint16_t cell_len)
+conflux_cell_parse_linked(const relay_msg_t *msg)
 {
   /* At the moment, same exact payload so avoid code duplication. */
-  return conflux_cell_parse_link(cell, cell_len);
+  return conflux_cell_parse_link(msg);
 }
 
 conflux_cell_link_t *
@@ -284,15 +284,15 @@ conflux_cell_new_link(const uint8_t *nonce, uint64_t last_seqno_sent,
  * Extracts the sequence number from a switch cell.
  */
 uint32_t
-conflux_cell_parse_switch(const cell_t *cell, uint16_t rh_len)
+conflux_cell_parse_switch(const relay_msg_t *msg)
 {
   uint32_t seq = 0;
   trn_cell_conflux_switch_t *switch_cell = NULL;
-  tor_assert(cell);
+
+  tor_assert(msg);
 
   if (trn_cell_conflux_switch_parse(&switch_cell,
-                                    cell->payload + RELAY_HEADER_SIZE,
-                                    rh_len) < 0) {
+                                    msg->body, msg->length) < 0) {
     log_warn(LD_BUG, "Failed to parse switch cell");
     // Zero counts as a failure to the validation, since legs should
     // not switch after 0 cells.
@@ -311,7 +311,7 @@ bool
 conflux_send_switch_command(circuit_t *send_circ, uint64_t relative_seq)
 {
   trn_cell_conflux_switch_t *switch_cell = trn_cell_conflux_switch_new();
-  cell_t cell;
+  uint8_t payload[RELAY_PAYLOAD_SIZE_MAX] = {0};
   bool ret = true;
 
   tor_assert(send_circ);
@@ -328,12 +328,12 @@ conflux_send_switch_command(circuit_t *send_circ, uint64_t relative_seq)
     goto end;
   }
 
-  memset(&cell, 0, sizeof(cell));
-
   trn_cell_conflux_switch_set_seqnum(switch_cell, (uint32_t)relative_seq);
 
-  if (trn_cell_conflux_switch_encode(cell.payload, RELAY_PAYLOAD_SIZE,
-                                     switch_cell) < 0) {
+  ssize_t len = trn_cell_conflux_switch_encode(
+                                payload, RELAY_PAYLOAD_SIZE_MAX,
+                                switch_cell);
+  if (len < 0) {
     log_warn(LD_BUG, "Failed to encode conflux switch cell");
     ret = false;
     goto end;
@@ -343,18 +343,16 @@ conflux_send_switch_command(circuit_t *send_circ, uint64_t relative_seq)
   if (CIRCUIT_IS_ORIGIN(send_circ)) {
     relay_send_command_from_edge(0, send_circ,
                                RELAY_COMMAND_CONFLUX_SWITCH,
-                               (const char*)cell.payload,
-                               RELAY_PAYLOAD_SIZE,
+                               (const char*)payload, len,
                                TO_ORIGIN_CIRCUIT(send_circ)->cpath->prev);
   } else {
     relay_send_command_from_edge(0, send_circ,
                                RELAY_COMMAND_CONFLUX_SWITCH,
-                               (const char*)cell.payload,
-                               RELAY_PAYLOAD_SIZE, NULL);
+                               (const char*)payload, len,
+                               NULL);
   }
 
 end:
   trn_cell_conflux_switch_free(switch_cell);
   return ret;
 }
-
