@@ -228,7 +228,7 @@ dirserv_load_fingerprint_file(void)
         add_status = RTR_INVALID;
     }  else if (!strcasecmp(nickname, "!middleonly")) {
         add_status = RTR_MIDDLEONLY;
-    } else if (!strcasecmp(nickname, "!stripexit")) {
+    } else if (!strcasecmp(nickname, "!stripguard")) {
         add_status = RTR_STRIPGUARD;
     } else if (!strcasecmp(nickname, "!striphsdir")) {
         add_status = RTR_STRIPHSDIR;
@@ -410,8 +410,8 @@ dirserv_rejects_tor_version(const char *platform,
   static const char please_upgrade_string[] =
     "Tor version is insecure or unsupported. Please upgrade!";
 
-  /* Anything before 0.4.8.0 is unsupported. Reject them. */
-  if (!tor_version_as_new_as(platform,"0.4.8.0-alpha-dev")) {
+  if (!tor_version_as_new_as(platform,
+        dirauth_get_options()->MinimalAcceptedServerVersion)) {
     if (msg) {
       *msg = please_upgrade_string;
     }
@@ -771,6 +771,16 @@ dirserv_add_descriptor(routerinfo_t *ri, const char **msg, const char *source)
   log_info(LD_DIR, "Assessing new descriptor: %s: %s",
            ri->nickname, ri->platform);
 
+  /* For now, TAP keys are still required. */
+  if (! ri->tap_onion_pkey) {
+    log_info(LD_DIRSERV, "Rejecting descriptor from %s (source: %s); "
+             "it has no TAP key.",
+             router_describe(ri), source);
+    *msg = "Missing TAP key in descriptor.";
+    r = ROUTER_AUTHDIR_REJECTS;
+    goto fail;
+  }
+
   /* Check whether this descriptor is semantically identical to the last one
    * from this server.  (We do this here and not in router_add_to_routerlist
    * because we want to be able to accept the newest router descriptor that
