@@ -113,7 +113,7 @@ typedef struct hs_service_intropoints_t {
   digest256map_t *map;
 
   /** Contains node's identity key digest that were introduction point for this
-   * descriptor but were retried to many times. We keep those so we avoid
+   * descriptor but were retried too many times. We keep those so we avoid
    * re-picking them over and over for a circuit retry period.
    * XXX: Once we have #22173, change this to only use ed25519 identity. */
   digestmap_t *failed_id;
@@ -389,6 +389,9 @@ hs_service_add_ephemeral_status_t
 hs_service_add_ephemeral(ed25519_secret_key_t *sk, smartlist_t *ports,
                          int max_streams_per_rdv_circuit,
                          int max_streams_close_circuit,
+                         int pow_defenses_enabled,
+                         uint32_t pow_queue_rate,
+                         uint32_t pow_queue_burst,
                          smartlist_t *auth_clients_v3, char **address_out);
 int hs_service_del_ephemeral(const char *address);
 
