@@ -1,7 +1,7 @@
 /* Copyright (c) 2001 Matej Pfajfar.
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
- * Copyright (c) 2007-2020, The Tor Project, Inc. */
+ * Copyright (c) 2007-2021, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
 /**
@@ -20,14 +20,11 @@ struct or_options_t;
 int router_orport_seems_reachable(
                                          const struct or_options_t *options,
                                          int family);
-int router_dirport_seems_reachable(
-                                         const struct or_options_t *options);
 
-void router_do_reachability_checks(int test_or, int test_dir);
+void router_do_reachability_checks(void);
 void router_perform_bandwidth_test(int num_circs, time_t now);
 
 void router_orport_found_reachable(int family);
-void router_dirport_found_reachable(void);
 
 void router_reset_reachability(void);
 
@@ -41,10 +38,8 @@ void router_reset_reachability(void);
   ((void)(opts), 0)
 
 static inline void
-router_do_reachability_checks(int test_or, int test_dir)
+router_do_reachability_checks(void)
 {
-  (void)test_or;
-  (void)test_dir;
   tor_assert_nonfatal_unreached();
 }
 static inline void
@@ -55,16 +50,16 @@ router_perform_bandwidth_test(int num_circs, time_t now)
   tor_assert_nonfatal_unreached();
 }
 static inline int
-inform_testing_reachability(void)
+inform_testing_reachability(const tor_addr_t *addr, uint16_t port)
 {
+  (void) addr;
+  (void) port;
   tor_assert_nonfatal_unreached();
   return 0;
 }
 
 #define router_orport_found_reachable() \
   STMT_NIL
-#define router_dirport_found_reachable() \
-  STMT_NIL
 
 #define router_reset_reachability() \
   STMT_NIL
