@@ -16,6 +16,9 @@
 #include "lib/cc/torint.h"
 
 #ifndef SYS_SECCOMP
+#ifdef HAVE_SIGNAL_H
+#include <signal.h>
+#endif
 
 /**
  * Used by SIGSYS signal handler to check if the signal was issued due to a
