@@ -21,6 +21,7 @@
 #include "core/or/circuitlist.h"
 #include "core/or/circuituse.h"
 #include "core/or/onion.h"
+#include "core/or/relay_msg.h"
 
 #include "core/or/cell_st.h"
 #include "core/or/cpath_build_state_st.h"
@@ -1278,7 +1279,7 @@ static void
 test_circuit_extend(void *arg)
 {
   (void)arg;
-  cell_t *cell = tor_malloc_zero(sizeof(cell_t));
+  relay_msg_t *msg = tor_malloc_zero(sizeof(relay_msg_t));
   channel_t *p_chan = tor_malloc_zero(sizeof(channel_t));
   or_circuit_t *or_circ = tor_malloc_zero(sizeof(or_circuit_t));
   circuit_t *circ = TO_CIRCUIT(or_circ);
@@ -1293,10 +1294,14 @@ test_circuit_extend(void *arg)
 
   setup_full_capture_of_logs(LOG_INFO);
 
+  msg->command = RELAY_COMMAND_EXTEND2;
+  NONSTRING uint8_t body[3] = "xyz";
+  msg->body = body;
+
 #ifndef ALL_BUGS_ARE_FATAL
   /* Circuit must be non-NULL */
   tor_capture_bugs_(1);
-  tt_int_op(circuit_extend(cell, NULL), OP_EQ, -1);
+  tt_int_op(circuit_extend(msg, NULL), OP_EQ, -1);
   tt_int_op(smartlist_len(tor_get_captured_bug_log_()), OP_EQ, 1);
   tt_str_op(smartlist_get(tor_get_captured_bug_log_(), 0), OP_EQ,
             "!(ASSERT_PREDICT_UNLIKELY_(!circ))");
@@ -1308,7 +1313,7 @@ test_circuit_extend(void *arg)
   tt_int_op(circuit_extend(NULL, circ), OP_EQ, -1);
   tt_int_op(smartlist_len(tor_get_captured_bug_log_()), OP_EQ, 1);
   tt_str_op(smartlist_get(tor_get_captured_bug_log_(), 0), OP_EQ,
-            "!(ASSERT_PREDICT_UNLIKELY_(!cell))");
+            "!(ASSERT_PREDICT_UNLIKELY_(!rmsg))");
   tor_end_capture_bugs_();
   mock_clean_saved_logs();
 
@@ -1324,13 +1329,13 @@ test_circuit_extend(void *arg)
 
   /* Clients can't extend */
   server = 0;
-  tt_int_op(circuit_extend(cell, circ), OP_EQ, -1);
+  tt_int_op(circuit_extend(msg, circ), OP_EQ, -1);
   expect_log_msg("Got an extend cell, but running as a client. Closing.\n");
   mock_clean_saved_logs();
 
   /* But servers can. Unpack the cell, but fail parsing. */
   server = 1;
-  tt_int_op(circuit_extend(cell, circ), OP_EQ, -1);
+  tt_int_op(circuit_extend(msg, circ), OP_EQ, -1);
   expect_log_msg("Can't parse extend cell. Closing circuit.\n");
   mock_clean_saved_logs();
 
@@ -1343,7 +1348,7 @@ test_circuit_extend(void *arg)
   mock_extend_cell_parse_result = 0;
   mock_extend_cell_parse_calls = 0;
 
-  tt_int_op(circuit_extend(cell, circ), OP_EQ, -1);
+  tt_int_op(circuit_extend(msg, circ), OP_EQ, -1);
   tt_int_op(mock_extend_cell_parse_calls, OP_EQ, 1);
   expect_log_msg(
     "Client asked me to extend without specifying an id_digest.\n");
@@ -1354,7 +1359,7 @@ test_circuit_extend(void *arg)
   memset(&mock_extend_cell_parse_cell_out.node_id, 0xAA,
              sizeof(mock_extend_cell_parse_cell_out.node_id));
 
-  tt_int_op(circuit_extend(cell, circ), OP_EQ, -1);
+  tt_int_op(circuit_extend(msg, circ), OP_EQ, -1);
   tt_int_op(mock_extend_cell_parse_calls, OP_EQ, 1);
   expect_log_msg("Client asked me to extend to a zero destination port "
                  "or unspecified address '[scrubbed]'.\n");
@@ -1368,7 +1373,7 @@ test_circuit_extend(void *arg)
 
 #ifndef ALL_BUGS_ARE_FATAL
   tor_capture_bugs_(1);
-  tt_int_op(circuit_extend(cell, circ), OP_EQ, -1);
+  tt_int_op(circuit_extend(msg, circ), OP_EQ, -1);
   tt_int_op(mock_extend_cell_parse_calls, OP_EQ, 1);
   tt_int_op(smartlist_len(tor_get_captured_bug_log_()), OP_EQ, 1);
   tt_str_op(smartlist_get(tor_get_captured_bug_log_(), 0), OP_EQ,
@@ -1389,7 +1394,7 @@ test_circuit_extend(void *arg)
   /* Test circuit not established, but don't launch another one */
   mock_channel_get_for_extend_launch_out = 0;
   mock_channel_get_for_extend_nchan = NULL;
-  tt_int_op(circuit_extend(cell, circ), OP_EQ, 0);
+  tt_int_op(circuit_extend(msg, circ), OP_EQ, 0);
   tt_int_op(mock_extend_cell_parse_calls, OP_EQ, 1);
   tt_int_op(mock_channel_get_for_extend_calls, OP_EQ, 1);
 
@@ -1409,7 +1414,7 @@ test_circuit_extend(void *arg)
   mock_channel_get_for_extend_launch_out = 1;
   mock_channel_get_for_extend_nchan = NULL;
   mock_channel_connect_nchan = fake_n_chan;
-  tt_int_op(circuit_extend(cell, circ), OP_EQ, 0);
+  tt_int_op(circuit_extend(msg, circ), OP_EQ, 0);
   tt_int_op(mock_extend_cell_parse_calls, OP_EQ, 1);
   tt_int_op(mock_channel_get_for_extend_calls, OP_EQ, 1);
   tt_int_op(mock_channel_connect_calls, OP_EQ, 1);
@@ -1433,7 +1438,7 @@ test_circuit_extend(void *arg)
   mock_channel_get_for_extend_nchan = fake_n_chan;
   mock_channel_connect_nchan = NULL;
   mock_circuit_deliver_create_cell_result = 0;
-  tt_int_op(circuit_extend(cell, circ), OP_EQ, 0);
+  tt_int_op(circuit_extend(msg, circ), OP_EQ, 0);
   tt_int_op(mock_extend_cell_parse_calls, OP_EQ, 1);
   tt_int_op(mock_channel_get_for_extend_calls, OP_EQ, 1);
   tt_int_op(mock_channel_connect_calls, OP_EQ, 0);
@@ -1456,7 +1461,7 @@ test_circuit_extend(void *arg)
   mock_channel_get_for_extend_nchan = fake_n_chan;
   mock_channel_connect_nchan = NULL;
   mock_circuit_deliver_create_cell_result = -1;
-  tt_int_op(circuit_extend(cell, circ), OP_EQ, -1);
+  tt_int_op(circuit_extend(msg, circ), OP_EQ, -1);
   tt_int_op(mock_extend_cell_parse_calls, OP_EQ, 1);
   tt_int_op(mock_channel_get_for_extend_calls, OP_EQ, 1);
   tt_int_op(mock_channel_connect_calls, OP_EQ, 0);
@@ -1502,7 +1507,7 @@ test_circuit_extend(void *arg)
   mock_circuit_deliver_create_cell_calls = 0;
   mock_circuit_deliver_create_cell_result = 0;
 
-  tor_free(cell);
+  relay_msg_free(msg);
   /* circ and or_circ are the same object */
   tor_free(circ->n_hop);
   tor_free(circ->n_chan_create_cell);
@@ -1527,6 +1532,7 @@ test_onionskin_answer(void *arg)
   /* Circuit must be non-NULL */
   tor_capture_bugs_(1);
   tt_int_op(onionskin_answer(NULL, created_cell,
+                             RELAY_CRYPTO_ALG_TOR1,
                              keys, CPATH_KEY_MATERIAL_LEN,
                              rend_circ_nonce), OP_EQ, -1);
   tt_int_op(smartlist_len(tor_get_captured_bug_log_()), OP_EQ, 1);
@@ -1538,6 +1544,7 @@ test_onionskin_answer(void *arg)
   /* Created cell must be non-NULL */
   tor_capture_bugs_(1);
   tt_int_op(onionskin_answer(or_circ, NULL,
+                             RELAY_CRYPTO_ALG_TOR1,
                              keys, CPATH_KEY_MATERIAL_LEN,
                              rend_circ_nonce), OP_EQ, -1);
   tt_int_op(smartlist_len(tor_get_captured_bug_log_()), OP_EQ, 1);
@@ -1549,6 +1556,7 @@ test_onionskin_answer(void *arg)
   /* Keys must be non-NULL */
   tor_capture_bugs_(1);
   tt_int_op(onionskin_answer(or_circ, created_cell,
+                             RELAY_CRYPTO_ALG_TOR1,
                              NULL, CPATH_KEY_MATERIAL_LEN,
                              rend_circ_nonce), OP_EQ, -1);
   tt_int_op(smartlist_len(tor_get_captured_bug_log_()), OP_EQ, 1);
@@ -1560,6 +1568,7 @@ test_onionskin_answer(void *arg)
   /* The rend circuit nonce must be non-NULL */
   tor_capture_bugs_(1);
   tt_int_op(onionskin_answer(or_circ, created_cell,
+                             RELAY_CRYPTO_ALG_TOR1,
                              keys, CPATH_KEY_MATERIAL_LEN,
                              NULL), OP_EQ, -1);
   tt_int_op(smartlist_len(tor_get_captured_bug_log_()), OP_EQ, 1);
@@ -1574,6 +1583,7 @@ test_onionskin_answer(void *arg)
 
   /* Fail when formatting the created cell */
   tt_int_op(onionskin_answer(or_circ, created_cell,
+                             RELAY_CRYPTO_ALG_TOR1,
                              keys, CPATH_KEY_MATERIAL_LEN,
                              rend_circ_nonce), OP_EQ, -1);
   expect_log_msg("couldn't format created cell (type=0, len=0).\n");
