@@ -58,7 +58,8 @@ test_geoip(void *arg)
       "dirreq-stats-end 2010-08-12 13:27:30 (86400 s)\n"
       "dirreq-v3-ips ab=8\n"
       "dirreq-v3-reqs ab=8\n"
-      "dirreq-v3-resp ok=0,not-enough-sigs=0,unavailable=0,not-found=0,"
+      "dirreq-v3-resp "
+          "served=0,ok=0,not-enough-sigs=0,unavailable=0,not-found=0,"
           "not-modified=0,busy=0\n"
       "dirreq-v3-direct-dl complete=0,timeout=0,running=0\n"
       "dirreq-v3-tunneled-dl complete=0,timeout=0,running=0\n",
@@ -66,7 +67,8 @@ test_geoip(void *arg)
       "dirreq-stats-end 2010-08-12 13:27:30 (86400 s)\n"
       "dirreq-v3-ips \n"
       "dirreq-v3-reqs \n"
-      "dirreq-v3-resp ok=0,not-enough-sigs=0,unavailable=0,not-found=0,"
+      "dirreq-v3-resp "
+          "served=0,ok=0,not-enough-sigs=0,unavailable=0,not-found=0,"
           "not-modified=0,busy=0\n"
       "dirreq-v3-direct-dl complete=0,timeout=0,running=0\n"
       "dirreq-v3-tunneled-dl complete=0,timeout=0,running=0\n",
@@ -74,7 +76,8 @@ test_geoip(void *arg)
       "dirreq-stats-end 2010-08-12 13:27:30 (86400 s)\n"
       "dirreq-v3-ips \n"
       "dirreq-v3-reqs \n"
-      "dirreq-v3-resp ok=8,not-enough-sigs=0,unavailable=0,not-found=0,"
+      "dirreq-v3-resp "
+          "served=8,ok=8,not-enough-sigs=0,unavailable=0,not-found=0,"
           "not-modified=0,busy=0\n"
       "dirreq-v3-direct-dl complete=0,timeout=0,running=0\n"
       "dirreq-v3-tunneled-dl complete=0,timeout=0,running=0\n",
@@ -82,7 +85,8 @@ test_geoip(void *arg)
       "dirreq-stats-end 2010-08-12 13:27:30 (86400 s)\n"
       "dirreq-v3-ips \n"
       "dirreq-v3-reqs \n"
-      "dirreq-v3-resp ok=8,not-enough-sigs=0,unavailable=0,not-found=0,"
+      "dirreq-v3-resp "
+          "served=8,ok=8,not-enough-sigs=0,unavailable=0,not-found=0,"
           "not-modified=0,busy=0\n"
       "dirreq-v3-direct-dl complete=0,timeout=0,running=0\n"
       "dirreq-v3-tunneled-dl complete=0,timeout=0,running=4\n",
@@ -230,6 +234,7 @@ test_geoip(void *arg)
 
   /* Note a successful network status response and make sure that it
    * appears in the history string. */
+  geoip_note_ns_response(GEOIP_SERVED);
   geoip_note_ns_response(GEOIP_SUCCESS);
   s = geoip_format_dirreq_stats(now + 86400);
   tt_str_op(dirreq_stats_3,OP_EQ, s);
