@@ -385,7 +385,7 @@ do_resolve(const char *hostname,
   tor_addr_make_unspec(result_addr);
   *result_hostname = NULL;
 
-  s = tor_open_socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
+  s = tor_open_socket(sockshost->family,SOCK_STREAM,IPPROTO_TCP);
   if (s<0) {
     log_sock_error("creating_socket", -1);
     return -1;
